/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.labelprovider;

import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.wb.swt.SWTResourceManager;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IDerivedAnnotation;
import org.fortiss.tooling.base.ui.annotation.AnnotationEntry;
import org.fortiss.tooling.base.ui.annotation.editingsupport.ComboBoxEditingSupport;
import org.fortiss.tooling.base.ui.annotation.editingsupport.MultiSelectionEditingSupport;
import org.fortiss.tooling.base.ui.annotation.view.IAnnotationViewPart;

/**
 * This class provides the labels in the table of the annotation view. The labels are shown in
 * {@link TableViewer}-based annotation views when no item is selected for editing.
 * 
 * @author diewald, barner
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 8AF4DEFCE823DA131ABEFB84D93812C8
 */
public class AnnotationLabelProvider extends LabelProviderBase {
	/** Specification class of this column */
	protected Class<? extends IAnnotatedSpecification> specClass;

	/** String holding the key of which identifies the value to be displayed */
	protected String instanceKey;

	/**
	 * {@link EditingSupport} of the column to for which this {@link AnnotationLabelProvider}
	 * provides the label.
	 */
	EditingSupport editingSupport;

	/**
	 * Creates an {@link AnnotationLabelProvider} for a given annotation class.
	 * 
	 * @param specClass
	 *            specification which provides the values to be displayed
	 * 
	 * @param viewPart
	 *            view required to retrieve the selected elements
	 * @param instanceKey
	 */
	public AnnotationLabelProvider(Class<? extends IAnnotatedSpecification> specClass,
			IAnnotationViewPart viewPart, String instanceKey, EditingSupport editingSupport) {
		super(viewPart);
		this.specClass = specClass;
		this.instanceKey = instanceKey;
		this.editingSupport = editingSupport;
	}

	/** {@inheritDoc} */
	@Override
	public String getText(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry annotationEntry = (AnnotationEntry)element;

			Object value = null;
			try {
				value = annotationEntry.getSpecificationValue(specClass, instanceKey);
			} catch(Exception e) {
				// Ignore. Return "" in this case.
			}
			if(value != null) {
				if(editingSupport instanceof ComboBoxEditingSupport) {
					return ((ComboBoxEditingSupport)editingSupport).getLabel(element);
				} else if(editingSupport instanceof MultiSelectionEditingSupport) {
					return ((MultiSelectionEditingSupport)editingSupport).getLabel(element);
				}

				return value.toString();
			}
		}
		return "";
	}

	/** {@inheritDoc} */
	@Override
	public Color getBackground(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry annotationEntry = (AnnotationEntry)element;
			if(annotationEntry.getModelElement().equals(viewPart.getCurrentlySelectedObject())) {
				return super.getBackground(element);
			}

			if(!annotationEntry.canEdit(specClass, instanceKey)) {
				if(IDerivedAnnotation.class.isAssignableFrom(specClass) &&
						annotationEntry.getSpecificationValue(specClass, instanceKey) != null) {
					return SWTResourceManager.getColor(200, 240, 250);
				}
				return SWTResourceManager.getColor(230, 230, 230);
			}

		}
		return super.getBackground(element);
	}
}
