/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.conqat.lib.commons.collections.Pair;
import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.ecore.xml.type.AnyType;
import org.fortiss.tooling.kernel.ToolingKernelActivator;

/**
 * Utility methods used for Eclipse resource storage.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 3E61B8F07E25B4B4B9A96672DC35DDCB
 */
public final class EMFResourceUtils {
	/**
	 * Stores the given EObject in the Eclipse workspace creating the given
	 * IFile.
	 * 
	 * @param file
	 *            the target file of path <code>/project/path_to_file</code>
	 * @param modelRootElement
	 *            the root element of the model to be stored
	 */
	public static void createNewEclipseWorkspaceProject(IFile file, EObject modelRootElement)
			throws IOException {
		Resource r =
				new ResourceSetImpl().createResource(URI.createPlatformResourceURI(file
						.getFullPath().toString(), true));
		r.getContents().add(modelRootElement);
		r.save(buildOptionsMap());
		// TODO: A.2
		// FIXME: check if this is executed during SmokeComponentLibraryTest
	}

	/**
	 * Build the map of options used for storage of EMF models.
	 * 
	 * @return the default option map used by the kernel
	 */
	public static Map<String, Object> buildOptionsMap() {
		Map<String, Object> options = new HashMap<String, Object>();
		options.put(XMLResource.OPTION_ENCODING, "UTF-8");
		options.put(XMLResource.OPTION_RECORD_UNKNOWN_FEATURE, true);
		options.put(Resource.OPTION_ZIP, false);
		return options;
	}

	/**
	 * Loads the root model element from the given URI. Logs errors to the
	 * console and returns <code>null</code>.
	 * 
	 * @param uri
	 *            the resource URI
	 * @return the root {@link EObject}
	 */
	public static Pair<EObject, Map<EObject, AnyType>>
			loadModelFromFileWithUnknownFeatures(URI uri) {
		try {
			ResourceSet rset = new ResourceSetImpl();
			Resource r;
			r = rset.createResource(uri);
			r.load(EMFResourceUtils.buildOptionsMap());
			Map<EObject, AnyType> unknownFeatures =
					r instanceof XMIResource ? ((XMIResource)r).getEObjectToExtensionMap()
							: new HashMap<EObject, AnyType>();
			return new Pair<EObject, Map<EObject, AnyType>>(r.getContents().get(0), unknownFeatures);
		} catch(IOException ex) {
			// TODO: B.19
			LoggingUtils.error(ToolingKernelActivator.getDefault(), "Failed to load model from " +
					uri.toString(), ex);
		}
		return null;
	}

	/**
	 * Loads the root model element from the given URI. Logs errors to the
	 * console and returns <code>null</code>.
	 * 
	 * @param uri
	 *            the resource URI
	 * @return the root {@link EObject}
	 */
	public static EObject loadModelFromFile(URI uri) {
		return loadModelFromFileWithUnknownFeatures(uri).getFirst();
	}
}
