/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.forms;

import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.ui.forms.SectionPart;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.editor.FormPage;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

/**
 * Base class for sections in forms-based editors.
 * 
 * @author deissenb
 * @author $Author: beller $
 * @version $Rev: 34752 $
 * @levd.rating GREEN Hash: 881C9B5673434A64B3764F74EFAEE8DF
 */
public abstract class FormSectionBase extends SectionPart {

	/** The toolkit. */
	protected final FormToolkit toolkit;

	/** The section client. */
	protected final Composite sectionClient;

	/** The page this section belongs to. */
	private FormPage page;

	/**
	 * Create new section.
	 * 
	 * @param page
	 *            the page the section belongs to.
	 * @param title
	 *            section title
	 * @param description
	 *            section description (may be <code>null</code>)
	 * @param layout
	 *            the layout of the section client.
	 */
	public FormSectionBase(FormPage page, String title, String description,
			Layout layout) {
		super(page.getManagedForm().getForm().getBody(), page.getManagedForm()
				.getToolkit(), Section.DESCRIPTION
				| ExpandableComposite.TWISTIE | ExpandableComposite.TITLE_BAR
				| ExpandableComposite.EXPANDED);
		this.page = page;
		this.toolkit = page.getManagedForm().getToolkit();

		getSection().setText(title);
		if (description != null) {
			getSection().setDescription(description);
		}

		sectionClient = toolkit.createComposite(getSection());
		sectionClient.setLayout(layout);
		getSection().setClient(sectionClient);
	}

	/** Must be called to create the client. */
	protected void createClient() {
		createContent();
		toolkit.paintBordersFor(sectionClient);
	}

	/** Trigger reflow for expansion sate change. */
	@Override
	protected void expansionStateChanged(boolean expanded) {
		getManagedForm().getForm().reflow(true);
	}

	/** Get page the section belongs to. */
	protected FormPage getPage() {
		return page;
	}

	/** Get editor the section belongs to. */
	protected FormEditor getEditor() {
		return page.getEditor();
	}

	/**
	 * Template method for creating the content of the section. Note: Initially
	 * this was called automatically from the constructor. However this violate
	 * the rule not to call overridable method from a constructor and, indeed,
	 * create very strange problems with halfway constructed objects.
	 */
	protected abstract void createContent();
}